package com.yanzhenjie.andserver.sample.controller;

import android.annotation.SuppressLint;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.le.BluetoothLeScanner;
import android.graphics.Color;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import com.yanzhenjie.andserver.annotation.GetMapping;
import com.yanzhenjie.andserver.annotation.PathVariable;
import com.yanzhenjie.andserver.annotation.PostMapping;
import com.yanzhenjie.andserver.annotation.RequestBody;
import com.yanzhenjie.andserver.annotation.RequestMapping;
import com.yanzhenjie.andserver.annotation.RequestMethod;
import com.yanzhenjie.andserver.annotation.RequestParam;
import com.yanzhenjie.andserver.annotation.RestController;
import com.yanzhenjie.andserver.sample.MainActivity;
import com.yanzhenjie.andserver.sample.model.ConnectBluetoothRequestParam;
import com.yuanxu.ecg.L;
import com.yuanxu.ecg.bean.UserInfo;
import com.yanzhenjie.andserver.sample.model.ScannedDevice;
import com.yanzhenjie.andserver.sample.util.Constant;
import com.yanzhenjie.andserver.sample.util.FileUtils;
import com.yanzhenjie.andserver.sample.util.Result;
import com.yanzhenjie.andserver.sample.util.UtilsSharedPreference;
import com.yanzhenjie.andserver.util.MediaType;
import com.yuanxu.ecg.ECGManager;
import com.yuanxu.ecg.callback.BaseCallback;
import com.yuanxu.ecg.callback.ExecuteCallback;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.slf4j.helpers.NOPLogger;

import java.util.Collection;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.validation.Valid;

import cn.jesse.nativelogger.NLogger;
import io.mega.megablelib.MegaAdvParse;
import io.mega.megablelib.MegaBleBuilder;
import io.mega.megablelib.MegaBleCallback;
import io.mega.megablelib.MegaBleClient;
import io.mega.megablelib.MegaBleConst;
import io.mega.megablelib.enums.MegaBleBattery;
import io.mega.megablelib.model.MegaBleDevice;
import io.mega.megablelib.model.bean.MegaAdvOnly;
import io.mega.megablelib.model.bean.MegaAdvertisingParsedEntity;
import io.mega.megablelib.model.bean.MegaBleHeartBeat;
import io.mega.megablelib.model.bean.MegaV2LiveSpoLive;
import io.mega.megablelib.model.bean.MegaV2LiveSpoMonitor;
import io.mega.megablelib.model.bean.MegaV2Mode;
import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;

// Created by Develoven on 2023/4/23.
// Copyright (c) 2023 redflow. All rights reserved.
//
@RestController
@RequestMapping("/common")
public class CommonController extends BaseController{

    private static final String TAG = "CommonController";
    private final Long  SCAN_PERIOD = 10L;

    private static BluetoothAdapter mBluetoothAdapter;
    private static BluetoothLeScanner mScanner = null;
    //    private static BluetoothAdapter.LeScanCallback mLeScanCallback ;
    private static HashMap<String,Object> scannedDeviceList = new HashMap<>();
    int onProcess = 0;
    private String fileName;
    MegaBleClient megaBleClient ;
    static {
//        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        mBluetoothAdapter = MainActivity.mBluetoothAdapter;
        if(mBluetoothAdapter == null){
            Log.e(TAG,"current device not support bluetooth");
            throw new RuntimeException("current device not support bluetooth");
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            mScanner = mBluetoothAdapter.getBluetoothLeScanner();
        }

    }

    /**
     * open blue tooth :http://192.168.137.168:8080/common/open/bluetooth
     * @return
     */
    @GetMapping(value = "/open/bluetooth",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result openBluetooth(){
        Log.d(TAG, fileName == null?"fileName == null":fileName);
        //1.open bluetooth
        if (mBluetoothAdapter == null) {
            Log.e(TAG, "--------------- current device not support bluetooth ");
            return fail(null,"current device not support bluetooth");
        }
        if ( !mBluetoothAdapter.isEnabled()) {
            boolean res = mBluetoothAdapter.enable();
            if (!res) {
                Log.e(TAG, "--------------- bluetooth open failed ");
                return fail(null,"bluetooth open failed");
            }
        } else if (mBluetoothAdapter.isEnabled()) {
            Log.e(TAG, "--------------- bluetooth open success");
            return fail(null,"bluetooth open success");
        }

        return success(null,"bluetooth open success");

    }
    /**
     * close bluetooth : http://192.168.137.168:8080/common/close/bluetooth
     * @return
     */
    @GetMapping(value = "/close/bluetooth",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result closeBluetooth(){
        //1.close bluetooth
        if (mBluetoothAdapter == null) {
            Log.e(TAG, "--------------- current device not support bluetooth ");
            return fail(null,"current device not support bluetooth");
        }
        if ( mBluetoothAdapter.isEnabled()) {
            boolean res = mBluetoothAdapter.disable();
            if (!res) {
                Log.e(TAG, "--------------- bluetooth close failed ");
                return fail(null,"bluetooth close failed");
            }
        } else if (!mBluetoothAdapter.isEnabled()) {
            Log.e(TAG, "--------------- bluetooth close success");
            return fail(null,"bluetooth close success");
        }
        return success(null,"bluetooth close success");
    }

    /**
     * check bluetooth status : http://192.168.137.168:8080/common/status/bluetooth
     * @return
     */

    @GetMapping(value = "/status/bluetooth",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result checkBluetoothStatus(){
        if (mBluetoothAdapter == null) {
            Log.e(TAG, "--------------- current device not support bluetooth ");
            return fail(null,"current device not support bluetooth");
        }else {
            return success(mBluetoothAdapter.isEnabled());
        }
    }
    /**
     * scan device by model
     * model = PULSE|ECG
     * @return
     */
    @SuppressLint("CheckResult")
    @GetMapping(value = "/scan/device/{model}",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result scanDevice(@PathVariable("model") String model) throws InterruptedException {
        Log.d(TAG, fileName == null?"fileName == null":fileName);
        AtomicBoolean scanStatus = new AtomicBoolean(true);
        BluetoothAdapter.LeScanCallback mLeScanCallback = (device, rssi, scanRecord) -> {
            try{

                Log.d(TAG+"LeScan","device:{"+device.getName()+"  "+rssi+"   "+scanRecord);
                if(StringUtils.isEmpty(device.getName())) {
                    return;
                }
                if(model.equals(Constant.MODEL_PULSE)){
                    if (!device.getName().toLowerCase().contains("ring")) return;
                    if(scanRecord.length < 62) return;
                    // parse 3rd ring
                    MegaAdvOnly advOnly = MegaAdvParse.parse(scanRecord);
                    //parse 2nd ring
                    if(advOnly == null){
                        MegaAdvertisingParsedEntity advEntity = MegaBleClient.parseScanRecord(device, scanRecord);
                        if(advEntity != null){
                            Log.d(TAG,advEntity.toString());
                        }
                    }else {
                        Log.d(TAG,advOnly.toString());
                    }
                    ScannedDevice scannedDevice = new ScannedDevice(device.getName(), device.getAddress(), rssi);
                    Log.d(TAG, "device: "+scannedDevice);
                    scannedDeviceList.put(scannedDevice.getAddress(),scannedDevice);
                }else if(model.equals(Constant.MODEL_ECG)){
                    if (!device.getName().toLowerCase().contains("bw-ecg")) return;
                    scannedDeviceList.put(device.getAddress(),device.getAddress());
                }

            }catch (Exception e ){
                Log.d(TAG,e.getMessage());
                e.printStackTrace();
            }
        };
        Log.d(TAG,"model="+model);
        //scan PULSE device
        Log.d(TAG,"size---"+scannedDeviceList.size());
        if(scannedDeviceList.size()>0){
            scannedDeviceList.clear();
        }
        Log.d(TAG,"size---"+scannedDeviceList.size());
        //scan ecg device
        Observable.timer(SCAN_PERIOD,TimeUnit.SECONDS).subscribe( aLong -> {
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
            scanStatus.set(false);
            Log.d(TAG,"scannedDeviceList.size---"+scannedDeviceList.size());
        });
        Log.d(TAG,"mLeScanCallback---"+mLeScanCallback);
        Log.d(TAG,"mBluetoothAdapter---"+mBluetoothAdapter);
        Log.i(TAG,"开始扫描");
        mBluetoothAdapter.startLeScan(mLeScanCallback);
        Observable.interval(1,TimeUnit.SECONDS).take(SCAN_PERIOD).observeOn(AndroidSchedulers.mainThread()).subscribe(aLong ->{
            Log.d(TAG,aLong.toString()+"scanning...");

        });
        while (scanStatus.get()){
            Thread.sleep(1000);
            Log.d(TAG, scanStatus+"---"+System.currentTimeMillis()+"----");
            Log.i(TAG,"等待");
        }

        return success(scannedDeviceList.values());
    }

    /**
     * connect bluetooth
     * @return
     */

    @PostMapping(value ="/connect/bluetooth/{model}",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result<Object> connectBluetooth( @PathVariable("model") String model,@RequestBody ConnectBluetoothRequestParam param){

        Log.i(TAG,param.getMac());
        fileName = "测试文件名称";

        if(model.equals(Constant.MODEL_PULSE)){
            fileName = "脉诊-"+ param.getUserName() +"-"+ param.getPhoneNumber()+"-"+DateFormatUtils.format(System.currentTimeMillis(), "yyyyMMddHHmmss")+".txt";
            if(StringUtils.isEmpty(param.getName()))
                return fail(null, "null param exception: param name is null");
            MegaBleCallback megaBleCallback = new MegaBleCallback() {
                @Override
                public void onConnectionStateChange(boolean connected, MegaBleDevice device) {
                    super.onConnectionStateChange(connected, device);
                    //状态变更 ：通知前台服务
                    if(connected){
                        Log.d(TAG, "已连接："+device);
                    }else {
                        Log.d(TAG, "断开连接");
                    }
                }
                @Override
                public void onError(int code) {
                    NLogger.e(TAG,"error code :{}",code);
                }

                @Override
                public void onStart() {
                    Log.d(TAG, "onStart:--");
                    String userId = "5837288dc59e0d00577c5f9a";
                    String token = UtilsSharedPreference.get(MainActivity.context,UtilsSharedPreference.KEY_TOKEN);
                    if(StringUtils.isEmpty(token)){
                        megaBleClient.startWithToken(userId, "0,0,0,0,0,0");
                    }else {
                        megaBleClient.startWithToken(userId, token);
                    }
                }

                @Override
                public void onDeviceInfoReceived(MegaBleDevice device) {
                    Log.d(TAG, "onDeviceInfoReceived----");
                    megaBleClient.getV2Batt();
                    megaBleClient.getV2Mode();
                    megaBleClient.enableV2ModeSpoMonitor(true);
                }
                @Override
                public void onSetUserInfo() {
                    megaBleClient.setUserInfo((byte) param.getAge(), (byte) param.getGender(), (byte) param.getHeight(), (byte) param.getWeight(), (byte) 0);
                }

                @Override
                public void onIdle() {
                    // 设备闲时，可开启实时、开启长时监控、收监控数据。
                    // 长时监控数据会被记录到戒指内部，实时数据不会。
                    // 长时监控开启后，可断开蓝牙连接，戒指将自动保存心率血氧数据，以便后续手机连上收取。默认每次连上会同步过来。
                    // 绑定token有变动时，用户信息，监测数据将被清空。
                    // 建议默认开启全局实时通道，无需关闭，重复开启无影响
                    // suggested setting, repeated call is ok.
                    Log.d(TAG, "Important: the remote device is idle.");
                    megaBleClient.toggleLive(true);
                }

                @Override
                public void onV2LiveSpoLive(MegaV2LiveSpoLive live) {

                }

                @Override
                public void onV2LiveSpoMonitor(MegaV2LiveSpoMonitor live) {
               /* switch (live.getStatus()){
                    case MegaBleConst.STATUS_LIVE_VALID:
                        updateV2Live(live);
                        break;
                    case MegaBleConst.STATUS_LIVE_PREPARING
                        updateV2Live(live);
                    break;
                    case MegaBleConst.STATUS_LIVE_INVALID
                            updateV2Live(live);
                    break;

                }*/
                }

                //notice user to shaking ,in order to bind the ring
                @Override
                public void onKnockDevice() {

                }

                @Override
                public void onTokenReceived(String token) {
                    UtilsSharedPreference.put(MainActivity.context, UtilsSharedPreference.KEY_TOKEN, token);
                }

                //notice UI modify rssi
                @Override
                public void onRssiReceived(int rssi) {

                }

                /**
                 * notify front to change battery value & battery status
                 *
                 * @param value 电池电量
                 * @param status 电池状态  normal(0, "normal"), charging(1, "charging"),full(2, "full"),lowPower(3, "lowPower");error(4, "error");shutdown(5, "shutdown");
                 */
                @Override
                public void onBatteryChanged(int value, int status) {
//                MegaBleBattery.getDescription(status);
                }

                /**
                 * notify front active mode
                 * @param mode
                 */
                @Override
                public void onV2ModeReceived(MegaV2Mode mode) {

                }

                /**
                 * 心跳检测，检测到心跳就开启脉搏
                 * @param heartBeat
                 */
                @Override
                public void onHeartBeatReceived(MegaBleHeartBeat heartBeat) {

                }

                @Override
                public void onRawdataParsed(int[][] a) {
                    if(a != null && a.length > 0){
                        for (int[] ints : a) {
                            Log.d(TAG,"rawDataParsed:"+ints[0]);
                            Log.d(TAG, "filePath:"+MainActivity.context.getFilesDir().getPath()+"/PULSE");
                            FileUtils.addFile(MainActivity.context.getFilesDir().getPath() + "/PULSE/"+fileName, String.valueOf(ints[0]));
                        }
                    }
                }
            };
            // mock id, key，use yours
            MegaBleBuilder builder = new MegaBleBuilder();
            try{
                megaBleClient = builder
                        .withSecretId("D4CE5DD515F81247")
                        .withSecretKey("uedQ2MgVEFlsGIWSgofHYHNdZSyHmmJ5")
                        .withContext(MainActivity.context)
                        .withCallback(megaBleCallback)
                        .build();
            }catch (Exception e ){
                e.printStackTrace();
            }
            // 开发测试时，可以开启debug
            megaBleClient.setDebugEnable(true);
            megaBleClient.connect(param.getMac(), param.getName());
        }else if(model.equals(Constant.MODEL_ECG)){
            fileName =  "心电-"+param.getUserName()+"-"+param.getPhoneNumber()+"-"+DateFormatUtils.format(System.currentTimeMillis(), "yyyyMMddHHmmss");
            //connect  ECG device
            ECGManager
                    .getInstance()
                    .setLog(true, "ECG")
                    .setAutoReconnect(true)
                    .init(MainActivity.application);
            UserInfo ecgUserInfo = new UserInfo(param.getName(), param.getGender() == 0, param.getAge(), param.getHeight(), param.getWeight());
            ECGManager.getInstance().execute(param.getMac(), ecgUserInfo, new ExecuteCallback() {
                long from = 0;
                @Override
                public void onSuccess() {//本轮检测任务成功
                    L.e("onSuccess");
                }

                @Override
                public void onFailure(int failCode, String info) {//本轮检测任务失败
                    switch (failCode) {
                        case BaseCallback.FAIL_BLUETOOTH_NOT_AVAILABLE: //蓝牙不可用
                            break;
                        case BaseCallback.FAIL_OTHER:  //其他原因
                            break;
                        case ExecuteCallback.FAIL_CONNECTION_ALREADY_ESTABLISHED://本机已连接到其他硬件设备
                            break;
                        case ExecuteCallback.FAIL_CONNECTION_START_FAIL://连接未正常开始
                            break;
                    }
                    L.e("onFailure---" + failCode + "     info=" + info);
                }

                @SuppressLint("ResourceType")
                @Override
                public void onProcess(int process, String info) {//检测任务进度
                    onProcess = process;
                    switch (process) {
                        /**
                         *连接进度
                         */
                        case PROCESS_CONNECT_START://连接开始
                            break;
                        case PROCESS_CONNECT_FAIL://连接失败
                            break;
                        case PROCESS_CONNECTED://已连接
                        case PROCESS_DATA_RECEIVING://正在接收硬件发回的心电数据
                            break;
                        case PROCESS_DISCONNECTED://连接断开
                            break;

                        /**
                         * notify进度
                         */
                        case PROCESS_NOTIFY_START://开始notify
                            break;
                        case PROCESS_NOTIFY_SUCCESS://notify成功
                            break;
                        case PROCESS_NOTIFY_FAIL://notify失败
                            break;

                        /**
                         *  其他如指令收发、数据接收等进度
                         */
                        case PROCESS_CMD_SENDING://正在发送相关指令
                            from = System.currentTimeMillis();
                            break;
                        case PROCESS_IDLE://任务结束（成功或失败），达到空闲状态
                            break;
                    }
                    L.d("onProcess---" + process + "     info=" + info);
                }
                @RequiresApi(api = Build.VERSION_CODES.O)
                @Override
                public void onReceivedOriginalData(byte[] data) {//原始心电数据
                    //发送数据至前端页面,则不需要解析数据
                    //parseRealTimeData(data);

                }
            });

        }else{
            return fail(null, "wrong model");
        }

        return success();
    }
}
